# muscle::ZLibUtilityFunctions class [(API)](https://public.msli.com/lcs/muscle/html/group__zipfileutilityfunctions.html)

```#include "zlib/ZLibUtilityFunctions.h"```

The functions in [ZLibUtilityFunctions](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html) are convenience functions for quickly deflating or inflating data.

* [DeflateByteBuffer()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#gaf8903f6895fe83edf6c8858dfe53b660) takes a [ByteBufferRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ByteBuffer.html) of raw data and returns a corresponding [ByteBufferRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ByteBuffer.html) of deflated data
* [InflateByteBuffer()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#gacfda730f50ef44a12fb8e86ff5e19ec3) takes a [ByteBufferRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ByteBuffer.html) of deflated data (previously produced by [DeflateByteBuffer()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#gaf8903f6895fe83edf6c8858dfe53b660)) and returns a corresponding [ByteBufferRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ByteBuffer.html) of re-inflated data
* [DeflateMessage()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#ga68910ff4695e9dacb9065cebbf7de3d6) takes a [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) and returns a corresponding [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) containing only a single field of flattened data (but with the same [what](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html#afaa29abc90165cc887596fd8021e9795)-code as the original [Message](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html)) -- or if the [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) that was passed in was already deflated, it just returns the passed-in [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) verbatim.
* [InflateMessage()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#ga3d3dd28c066109200a7507446a83fb88) takes a [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) that was previously returned by [DeflateMessage()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#ga68910ff4695e9dacb9065cebbf7de3d6) and returns a corresponding re-inflated [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) containing all of the original fields again.  (or if the [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) that was passed in wasn't a deflated Message, it just returns the passed-in [MessageRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) verbatim)
* [ReadAndDeflateAndWrite()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#ga9e6c9414f55894bb3d9f0072ee21ff98) reads raw data from one [DataIO](https://public.msli.com/lcs/muscle/html/classmuscle_1_1DataIO.html), deflates it, and writes the deflated data out to another [DataIO](https://public.msli.com/lcs/muscle/html/classmuscle_1_1DataIO.html), without ever loading all of the data into RAM at once.  (Useful for deflating large files)
* [ReadAndInflateAndWrite()](https://public.msli.com/lcs/muscle/html/group__zlibutilityfunctions.html#ga793980b1325abd30eb7061af3b3c43d7) reads deflated data from one [DataIO](https://public.msli.com/lcs/muscle/html/classmuscle_1_1DataIO.html), inflates it, and writes the inflated data out to another [DataIO](https://public.msli.com/lcs/muscle/html/classmuscle_1_1DataIO.html), without ever loading all of the data into RAM at once.  (Useful for inflating large files)

Try compiling and running the mini-example-programs in `muscle/html/muscle-by-example/examples/zlibutilityfunctions` (enter `make` to compile example_*, and then run each from Terminal while looking at the corresponding .cpp file)

Quick links to source code of relevant MUSCLE-by-example programs:

* [zlibutilityfunctions/example_1_byte_buffers.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/zlibutilityfunctions/example_1_byte_buffers.cpp)
* [zlibutilityfunctions/example_2_messages.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/zlibutilityfunctions/example_2_messages.cpp)
