# muscle::PulseNode class [(API)](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html)

```#include "util/PulseNode.h"```

[PulseNode](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html) is an interface inherited by classes that may want to get their [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) callback-method called at various times they specify via their [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) method.

* An object that subclasses [PulseNode](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html) may reimplement its [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) method to return a time (in 64-bit [GetRunTime64()](https://public.msli.com/lcs/muscle/html/group__timeutilityfunctions.html#ga37279a5f229f0402eee6f83d696ea9aa)-style microseconds) at which its [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) method should be called next (or it can return [MUSCLE_TIME_NEVER](https://public.msli.com/lcs/muscle/html/group__timeutilityfunctions.html#gab8528514fe5484bda1132f96db91874d) if it doesn't currently want [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) to be called).
* At the scheduled time that was returned by [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) (or very shortly thereafter) [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) will be called.
* This only works if the object is attached to event loop whose [PulseNodeManager](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNodeManager.html) (typically the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html)) is implemented to perform the necessary [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) calls.
* If the object changes its mind about when [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) should be called next, it should call [InvalidatePulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afc567231b47be46a7d660eb8f79fb65a) to let the [PulseNodeManager](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNodeManager.html) know it needs to call [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) again to get the new next-pulse-scheduled-for time.  (Exception:  this isn't necessary inside [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) itself, since [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) will always be called after a call to [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d))
* Built-in classes that offer [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149)/[Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) callbacks include:  [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html), [ReflectSessionFactory](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectSessionFactory.html), [AbstractReflectSession](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html), [AbstractMessageIOGateway](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractMessageIOGateway.html), [AbstractSessionIOPolicy](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractSessionIOPolicy.html)
* When reimplementing [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) and [Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d), be sure to call up to the parent class's implementation of those methods in order not to break the parent-class's internal Pulse-callback-functionality.  (Combine your [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149) result-value with the parent's result-value via `muscleMin()`)
* If a subclass of [PulseNode](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html) wants to offer [PulseNodeManager](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNodeManager.html)-style [GetPulseTime()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#afb739573da69c2b540c9a921d732a149)/[Pulse()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#ac530185533f417b7c6c86032c82b6a0d) service to other objects that it owns, it can do so easily, just by subclassing those objects from [PulseNode](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html) as well, and calling [PutPulseChild()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNode.html#a95062a4b68e52b92a3b5963d7e13e8bd) on each object once, to add the object to its [PulseNodeManager](https://public.msli.com/lcs/muscle/html/classmuscle_1_1PulseNodeManager.html)'s callback-list.

Try compiling and running the mini-example-programs in `muscle/html/muscle-by-example/examples/reflector` (enter `make` to compile example_*, and then run each from Terminal while looking at the corresponding .cpp file)

Quick links to source code of relevant MUSCLE-by-example programs:

* [reflector/example_6_smart_server_with_pulsenode.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_6_smart_server_with_pulsenode.cpp)
* [reflector/example_7_smart_server_with_udp_pingpong.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_7_smart_server_with_udp_pingpong.cpp)
